# $Rev: 15 $
# $Date: 2008-07-10 17:52:40 -0500 (Thu, 10 Jul 2008) $
# Author: Michael Hennebry

class PpmError(StandardError) : pass
class BadMagicError(PpmError) : pass
class OutOfRangeError(PpmError) : pass


class Ppm :
    # constructor takes only an open file.
    # read is the only file attribute constructor uses.
    # instance variables:
    # width
    # height
    # max_level, maximum value of a component, must be in 1..1023
    # pixels, string rgbRGBrgb... from top row to bottom, from left to right,
    #         1 or 2 bytes per component
    #
    # ppm file format
    # P6 width height max_level binary_data
    # precisely 1 byte after max_level
    def __init__(self, phile) :
        magic=phile.read(2)
        if 'P6' != magic : raise BadMagicError
        ch=phile.read(1)
        while ch.isspace() : ch=phile.read(1)
        widthStr=''
        while ch.isdigit() :
            widthStr=widthStr+ch
            ch=phile.read(1)
        self.width=int(widthStr)
        if self.width< 0 : raise OutOfRangeError, ('width', self.width)

        while ch.isspace() : ch=phile.read(1)
        heightStr=''
        while ch.isdigit() :
            heightStr=heightStr+ch
            ch=phile.read(1)
        self.height=int(heightStr)
        if self.height< 0 : raise OutOfRangeError, ('height', self.height)

        while ch.isspace() : ch=phile.read(1)
        max_levelStr=''
        while ch.isdigit() :
            max_levelStr=max_levelStr+ch
            ch=phile.read(1)
        self.max_level=int(max_levelStr)
        if self.max_level< 0 or self.max_level> 0xFFFF :
            raise OutOfRangeError, ('max_level', self.max_level)

        self.bytes_per_component=1+(self.max_level> 0xFF);  # MSByte first
        print self.bytes_per_component, self.width, self.height
        ll=self.bytes_per_component*3*self.width*self.height
        left=ll
        self.bytes=''
        empties=0
        while left and not phile.closed and empties< 10 :
            chunk=phile.read(left)
            print "len(chunk)=", len(chunk)
            if chunk :
                self.bytes=self.bytes+chunk
                left-=len(chunk)
                empties=0
            else :
                empties+=1
        print ll, '=len(bytes)=', len(self.bytes), phile.closed, len(phile.read())
        for ch in self.bytes[:4] + self.bytes[-4:] :
            print '%02x' % ord(ch),
        print

    def fetch_row_bytes(y, x=0, width=None) :
        if 0> y or y>=self.width  : raise OutOfRangeError, ('y', y)
        if 0> x or x>=self.height : raise OutOfRangeError, ('x', x)
        if None==width : width=self.width-x
        if 0> width or width+x>=self.width :
            raise OutOfRangeError, ('width', width)
        return (self.bytes[3*self.bytes_per_component*(self.width*y+x):]
                         [:3*self.bytes_per_component*width] )

if __name__=="__main__" :
    x=Ppm(file("ff48x32.ppm"))
    x=Ppm(file("bw48x32.ppm"))
    x=Ppm(file("bisonfootball.ppm"))
